@@ -8,8 +8,8 @@ from kaizen.llms.provider import LLMProvider
 from kaizen.llms.prompts.pr_desc_prompts import (
     PR_DESCRIPTION_PROMPT,
     MERGE_PR_DESCRIPTION_PROMPT,
-    PR_FILE_DESCRIPTION_PROMPT,
     PR_DESCRIPTION_SYSTEM_PROMPT,
+    PR_COMMIT_MESSAGE_PROMPT,
 )
 
 
@@ -41,8 +41,6 @@ class PRDescriptionGenerator:
         user: Optional[str] = None,
     ) -> DescOutput:
         prompt = PR_DESCRIPTION_PROMPT.format(
-            PULL_REQUEST_TITLE=pull_request_title,
-            PULL_REQUEST_DESC=pull_request_desc,
             CODE_DIFF=diff_text,
         )
         if not diff_text and not pull_request_files:
@@ -99,9 +97,12 @@ class PRDescriptionGenerator:
         ):
             file_descs.extend(file_review)
 
-        prompt = MERGE_PR_DESCRIPTION_PROMPT.format(DESCS=json.dumps(file_descs))
-        resp, usage = self.provider.chat_completion(prompt, user=user)
-        desc = parser.extract_code_from_markdown(resp)
+        if len(file_descs) > 1:
+            prompt = MERGE_PR_DESCRIPTION_PROMPT.format(DESCS=json.dumps(file_descs))
+            resp, usage = self.provider.chat_completion(prompt, user=user)
+            desc = parser.extract_code_from_markdown(resp)
+        else:
+            desc = parser.extract_code_from_markdown(file_descs[0])
         self.total_usage = self.provider.update_usage(self.total_usage, usage)
 
         return desc
@@ -115,9 +116,7 @@ class PRDescriptionGenerator:
     ) -> Generator[List[Dict], None, None]:
         combined_diff_data = ""
         available_tokens = self.provider.available_tokens(
-            PR_FILE_DESCRIPTION_PROMPT.format(
-                PULL_REQUEST_TITLE=pull_request_title,
-                PULL_REQUEST_DESC=pull_request_desc,
+            PR_DESCRIPTION_PROMPT.format(
                 CODE_DIFF="",
             )
         )
@@ -164,9 +163,7 @@ class PRDescriptionGenerator:
         pull_request_desc: str,
         user: Optional[str],
     ) -> List[Dict]:
-        prompt = PR_FILE_DESCRIPTION_PROMPT.format(
-            PULL_REQUEST_TITLE=pull_request_title,
-            PULL_REQUEST_DESC=pull_request_desc,
+        prompt = PR_DESCRIPTION_PROMPT.format(
             CODE_DIFF=diff_data,
         )
         resp, usage = self.provider.chat_completion(prompt, user=user)
@@ -174,3 +171,14 @@ class PRDescriptionGenerator:
         self.total_usage = self.provider.update_usage(self.total_usage, usage)
 
         return desc
+
+    def generate_pr_commit_message(
+        self,
+        desc: str,
+        user: Optional[str] = None,
+    ) -> str:
+        prompt = PR_COMMIT_MESSAGE_PROMPT.format(
+            DESC=desc,
+        )
+        resp, usage = self.provider.chat_completion_with_json(prompt, user=user)
+        return resp, usage
