@@ -1,11 +1,11 @@
 import litellm
 import os
-from typing import Dict, Optional, Any
+from typing import Dict, Optional, Any, Tuple
 from kaizen.llms.prompts.general_prompts import BASIC_SYSTEM_PROMPT
 from kaizen.utils.config import ConfigData
 from kaizen.helpers.general import retry
 from kaizen.helpers.parser import extract_json
-from litellm import Router
+from litellm import Router, embedding
 import logging
 from collections import defaultdict
 
@@ -13,7 +13,7 @@ DEFAULT_MAX_TOKENS = 8000
 
 
 def set_all_loggers_to_ERROR():
-    print("All Loggers and their levels:")
+    # print("All Loggers and their levels:")
     for name, logger in logging.Logger.manager.loggerDict.items():
         if isinstance(logger, logging.Logger):
             # print(f"Logger: {name}, Level: {logging.getLevelName(logger.level)}")
@@ -83,6 +83,7 @@ class LLMProvider:
             "model_list": self.models,
             "allowed_fails": 1,
             "enable_pre_call_checks": True,
+            "routing_strategy": "simple-shuffle",
         }
 
         if self.config["language_model"].get("redis_enabled", False):
@@ -145,7 +146,7 @@ class LLMProvider:
         model="default",
         custom_model=None,
         messages=None,
-    ):
+    ) -> Tuple[str, Dict[str, int]]:
         if not messages:
             messages = [
                 {"role": "system", "content": self.system_prompt},
@@ -162,6 +163,32 @@ class LLMProvider:
         self.model = response["model"]
         return response["choices"][0]["message"]["content"], response["usage"]
 
+    def raw_chat_completion(
+        self,
+        prompt,
+        user: str = None,
+        model="default",
+        custom_model=None,
+        messages=None,
+        n_choices=1,
+    ) -> Tuple[Dict, Dict[str, int]]:
+        custom_model["n"] = n_choices
+        if not messages:
+            messages = [
+                {"role": "system", "content": self.system_prompt},
+                {"role": "user", "content": prompt},
+            ]
+        if not custom_model:
+            custom_model = {"model": model}
+        if "temperature" not in custom_model:
+            custom_model["temperature"] = self.default_temperature
+
+        response = self.provider.completion(
+            messages=messages, user=user, **custom_model
+        )
+        self.model = response["model"]
+        return response, response["usage"]
+
     @retry(max_attempts=3, delay=1)
     def chat_completion_with_json(
         self,
@@ -235,18 +262,24 @@ class LLMProvider:
             return {key: total_usage[key] + current_usage[key] for key in total_usage}
         return {key: current_usage[key] for key in current_usage}
 
-    def get_usage_cost(self, total_usage: Dict[str, int], model: str = None) -> float:
+    def get_usage_cost(
+        self, total_usage: Dict[str, int], model: str = None
+    ) -> Tuple[float, float]:
         if not model:
             model = self.model
-        return litellm.cost_per_token(
-            model, total_usage["prompt_tokens"], total_usage["completion_tokens"]
-        )
+        try:
+            return litellm.cost_per_token(
+                model, total_usage["prompt_tokens"], total_usage["completion_tokens"]
+            )
+        except Exception:
+            return 0, 0
 
     def get_text_embedding(self, text):
-        # for model in self.config["language_model"]["models"]:
-        #     if model["model_name"] == "embedding":
-        #         break
-        response = self.provider.embedding(
-            model="embedding", input=[text], dimensions=1536, encoding_format="float"
+        for model in self.config["language_model"]["models"]:
+            if model["model_name"] == "embedding":
+                model_name = model["litellm_params"]["model"]
+                break
+        response = embedding(
+            model=model_name, input=[text], dimensions=1536, encoding_format="float"
         )
         return response["data"], response["usage"]
