@@ -7,7 +7,12 @@ class JavaScriptParser:
         class_pattern = r"class\s+(\w+)\s*{([^}]*)}"
         function_pattern = r"(async\s+)?function\s+(\w+)\s*\((.*?)\)\s*{([^}]*)}"
 
+        import_pattern = r"import\s+.*?(?:from\s+['\"](.*?)['\"]|\s*;)"
+        global_var_pattern = r"(?:const|let|var)\s+(\w+)\s*=\s*([^;]+);"
+
         parsed_data = []
+        imports = re.findall(import_pattern, source)
+        global_vars = re.findall(global_var_pattern, source)
 
         for class_match in re.finditer(class_pattern, source):
             class_name = class_match.group(1)
@@ -27,6 +32,8 @@ class JavaScriptParser:
                         for m in methods
                     ],
                     "source": class_match.group(0),
+                    "imports": imports,
+                    "global_vars": global_vars,
                 }
             )
 
@@ -37,6 +44,8 @@ class JavaScriptParser:
                     "name": func_match.group(2),
                     "args": func_match.group(3).split(","),
                     "source": func_match.group(0),
+                    "imports": imports,
+                    "global_vars": global_vars,
                 }
             )
 
