@@ -1,12 +1,6 @@
 import os
 import logging
-from llama_index.core import (
-    StorageContext,
-    VectorStoreIndex,
-)
-from llama_index.core.schema import TextNode
-from kaizen.retriever.custom_vector_store import CustomPGVectorStore
-from sqlalchemy import create_engine, text
+from uuid import uuid4
 from llama_index.llms.litellm import LiteLLM
 import networkx as nx
 from typing import List, Dict, Any
@@ -16,8 +10,9 @@ from kaizen.llms.provider import LLMProvider
 from kaizen.retriever.code_chunker import chunk_code
 import traceback
 from llama_index.embeddings.litellm import LiteLLMEmbedding
-from llama_index.core import QueryBundle
-
+from sqlalchemy import create_engine, text
+from kaizen.retriever.qdrant_vector_store import QdrantVectorStore
+import json
 
 # Set up logging
 logging.basicConfig(
@@ -39,32 +34,33 @@ class RepositoryAnalyzer:
         )
         self.repo_id = repo_id
         self.graph = nx.DiGraph()
-        self.vector_store = CustomPGVectorStore.from_params(
-            database=os.environ["POSTGRES_DB"],
-            host=os.environ["POSTGRES_HOST"],
-            password=os.environ["POSTGRES_PASSWORD"],
-            port=os.environ["POSTGRES_PORT"],
-            user=os.environ["POSTGRES_USER"],
-            table_name="embeddings",
-            embed_dim=1536,
-        )
+        self.vector_store = QdrantVectorStore("embeddings", vector_size=1536)
         self.llm_provider = LLMProvider()
         self.llm = LiteLLM(model_name="small", router=self.llm_provider.provider)
         # embed_llm = LiteLLM(model_name="embedding", router=self.llm_provider.provider)
         self.embed_model = LiteLLMEmbedding(
             model_name="azure/text-embedding-3-small", router=self.llm_provider.provider
         )
-        self.storage_context = StorageContext.from_defaults(
-            vector_store=self.vector_store
-        )
         logger.info("RepositoryAnalyzer initialized successfully")
 
-    def setup_repository(self, repo_path: str):
+    def setup_repository(
+        self,
+        repo_path: str,
+        node_query: str = None,
+        file_query: str = None,
+        function_query: str = None,
+    ):
         self.total_usage = self.llm_provider.DEFAULT_USAGE
+        self.total_files_processed = 0
+        self.node_query = node_query
+        self.file_query = file_query
+        self.function_query = function_query
+        self.embedding_usage = {"prompt_tokens": 10, "total_tokens": 10}
         logger.info(f"Starting repository setup for: {repo_path}")
         self.parse_repository(repo_path)
         self.store_function_relationships()
         logger.info("Repository setup completed successfully")
+        return self.total_files_processed, self.total_usage, self.embed_model
 
     def parse_repository(self, repo_path: str):
         logger.info(f"Parsing repository: {repo_path}")
@@ -72,6 +68,7 @@ class RepositoryAnalyzer:
             futures = []
             for root, _, files in os.walk(repo_path):
                 for file in files:
+                    self.total_files_processed += 1
                     if file.endswith(
                         (".py", ".js", ".ts", ".rs")
                     ):  # Add more extensions as needed
@@ -128,8 +125,12 @@ class RepositoryAnalyzer:
 
         if isinstance(code_info, str):
             code = code_info
+            start_line = 1  # Default to 1 if no position information is available
         elif isinstance(code_info, dict) and "code" in code_info:
             code = code_info["code"]
+            start_line = code_info.get(
+                "start_line", 1
+            )  # Get start_line if available, default to 1
         else:
             logger.error(
                 f"Unexpected code_info format for {section} - {name}: {type(code_info)}"
@@ -137,9 +138,13 @@ class RepositoryAnalyzer:
             return  # Skip this code block
 
         language = self.get_language_from_extension(file_path)
-        abstraction, usage = self.generate_abstraction(code, language)
-
-        function_id = self.store_code_in_db(code, abstraction, file_path, section, name)
+        abstraction, usage = self.generate_abstraction(code, language, section)
+        self.total_usage = self.llm_provider.update_usage(
+            total_usage=self.total_usage, current_usage=usage
+        )
+        function_id = self.store_code_in_db(
+            code, abstraction, file_path, section, name, start_line
+        )
         self.store_abstraction_and_embedding(function_id, abstraction)
 
         if section == "functions":
@@ -152,6 +157,9 @@ class RepositoryAnalyzer:
         )
 
         embedding, emb_usage = self.llm_provider.get_text_embedding(abstraction)
+        self.embedding_usage = self.llm_provider.update_usage(
+            total_usage=self.embedding_usage, current_usage=emb_usage
+        )
         embedding = embedding[0]["embedding"]
         # Store the embedding in the database
         # TODO: DONT PUSH DUPLICATE
@@ -171,44 +179,99 @@ class RepositoryAnalyzer:
                 },
             )
 
-        # Create a TextNode for the vector store
-        # Include repo_id in the metadata
-        metadata = {"repo_id": self.repo_id}
-        node = TextNode(
-            text=abstraction,
-            id_=str(function_id),
-            embedding=embedding,
-            metadata=metadata,
-        )
+        # Create a dictionary instead of TextNode
+        node = {
+            "id": str(uuid4()),
+            "text": abstraction,
+            "embedding": embedding,
+            "metadata": {"repo_id": self.repo_id, "function_id": function_id},
+        }
 
-        # Add the node to the vector store
+        # Add the node to the vector store directly
         self.vector_store.add(nodes=[node])
 
         logger.debug(f"Abstraction and embedding stored for function_id: {function_id}")
 
     def generate_abstraction(
-        self, code_block: str, language: str, max_tokens: int = 300
+        self, code_block: str, language: str, section: str, max_tokens: int = 300
     ) -> str:
-        prompt = f"""Generate a concise yet comprehensive abstract description of the following {language} code block. 
-        Include information about:
-        1. The purpose or functionality of the code
-        2. Input parameters and return values (if applicable)
-        3. Any important algorithms or data structures used
-        4. Key dependencies or external libraries used
-        5. Any notable design patterns or architectural choices
-        6. Potential edge cases or error handling
+        prompt = f"""Analyze the following {language} code block and generate a structured abstraction. 
+Your response should be in JSON format and include the following sections:
 
-        Code:
-        ```{language}
-        {code_block}
-        ```
+{{
+  "summary": "A concise one-sentence summary of the function's primary purpose.",
+
+  "functionality": "A detailed explanation of what the function does, including its main steps and logic. Use multiple lines if needed for clarity.",
+
+  "inputs": [
+    {{
+      "name": "The parameter name",
+      "type": "The parameter type",
+      "description": "A brief description of the parameter's purpose",
+      "default_value": "The default value, if any (or null if not applicable)"
+    }}
+  ],
+
+  "output": {{
+    "type": "The return type of the function",
+    "description": "A description of what is returned and under what conditions. Use multiple lines if needed."
+  }},
+
+  "dependencies": [
+    {{
+      "name": "Name of the external library or module",
+      "purpose": "Brief explanation of its use in this function"
+    }}
+  ],
+
+  "algorithms": [
+    {{
+      "name": "Name of the algorithm or data structure",
+      "description": "Brief explanation of its use and importance"
+    }}
+  ],
+
+  "edge_cases": [
+    "A list of potential edge cases or special conditions the function handles or should handle"
+  ],
+
+  "error_handling": "A description of how errors are handled or propagated. Include specific error types if applicable.",
+
+  "usage_context": "A brief explanation of how this function might be used by parent functions or in a larger system. Include typical scenarios and any important considerations for its use.",
+
+  "complexity": {{
+    "time": "Estimated time complexity (e.g., O(n))",
+    "space": "Estimated space complexity (e.g., O(1))",
+    "explanation": "Brief explanation of the complexity analysis"
+  }},
+
+  "tags": ["List", "of", "relevant", "tags"],
+
+  "testing_considerations": "Suggestions for unit tests or test cases to cover key functionality and edge cases",
+
+  "version_compatibility": "Information about language versions or dependency versions this code is compatible with",
+
+  "performance_considerations": "Any notes on performance optimizations or potential bottlenecks",
+
+  "security_considerations": "Any security-related notes or best practices relevant to this code",
+
+  "maintainability_score": "A subjective score from 1-10 on how easy the code is to maintain, with a brief explanation"
+}}
+
+Provide your analysis in this clear, structured JSON format. If any section is not applicable, use an empty list [] or null value as appropriate. Ensure that multi-line descriptions are properly formatted as strings.
+
+Code to analyze:
+Language: {language}
+Block Type: {section}
+Code Block: 
+```{code_block}```
         """
 
         estimated_prompt_tokens = len(tokenizer.encode(prompt))
         adjusted_max_tokens = min(max(150, estimated_prompt_tokens), 1000)
 
         try:
-            abstraction, usage = self.llm_provider.chat_completion(
+            abstraction, usage = self.llm_provider.chat_completion_with_json(
                 prompt="",
                 messages=[
                     {
@@ -219,52 +282,32 @@ class RepositoryAnalyzer:
                 ],
                 custom_model={"max_tokens": adjusted_max_tokens, "model": "small"},
             )
-            return abstraction, usage
+            return json.dumps(abstraction), usage
 
         except Exception as e:
             raise e
 
-    def ensure_repository_exists(
-        self, repo_name, repo_owner, repo_url, repo_description=""
-    ):
-        with self.engine.begin() as connection:
-            repo_query = text(
-                """
-                INSERT INTO repositories (repo_name, repo_owner, repo_url, repo_description)
-                VALUES (:repo_name, :repo_owner, :repo_url, :repo_description)
-                ON CONFLICT (repo_name, repo_owner) DO UPDATE SET
-                    repo_url = EXCLUDED.repo_url,
-                    repo_description = EXCLUDED.repo_description
-                RETURNING repo_id
-            """
-            )
-            repo_id = connection.execute(
-                repo_query,
-                {
-                    "repo_name": repo_name,
-                    "repo_owner": repo_owner,
-                    "repo_url": repo_url,
-                    "repo_description": repo_description,
-                },
-            ).scalar_one()
-        return repo_id
-
     def store_code_in_db(
-        self, code: str, abstraction: str, file_path: str, section: str, name: str
+        self,
+        code: str,
+        abstraction: str,
+        file_path: str,
+        section: str,
+        name: str,
+        start_line: int,
     ) -> int:
         logger.debug(f"Storing code in DB: {file_path} - {section} - {name}")
         with self.engine.begin() as connection:
             # Insert into files table (assuming this part is already correct)
-            file_query = text(
-                """
-                INSERT INTO files (repo_id, file_path, file_name, file_ext, programming_language)
-                VALUES (:repo_id, :file_path, :file_name, :file_ext, :programming_language)
-                ON CONFLICT (repo_id, file_path) DO UPDATE SET file_path = EXCLUDED.file_path
-                RETURNING file_id
-                """
-            )
+            if not self.file_query:
+                self.file_query = """
+                        INSERT INTO files (repo_id, file_path, file_name, file_ext, programming_language)
+                    VALUES (:repo_id, :file_path, :file_name, :file_ext, :programming_language)
+                    ON CONFLICT (repo_id, file_path) DO UPDATE SET file_path = EXCLUDED.file_path
+                    RETURNING file_id
+                    """
             file_id = connection.execute(
-                file_query,
+                text(self.file_query),
                 {
                     "repo_id": self.repo_id,
                     "file_path": file_path,
@@ -275,23 +318,22 @@ class RepositoryAnalyzer:
             ).scalar_one()
 
             # Insert into function_abstractions table
-            function_query = text(
-                """
-                INSERT INTO function_abstractions 
-                (file_id, function_name, function_signature, abstract_functionality, start_line, end_line)
-                VALUES (:file_id, :function_name, :function_signature, :abstract_functionality, :start_line, :end_line)
-                RETURNING function_id
-                """
-            )
+            if not self.function_query:
+                self.function_query = """
+                    INSERT INTO function_abstractions 
+                    (file_id, function_name, function_signature, abstract_functionality, start_line, end_line)
+                    VALUES (:file_id, :function_name, :function_signature, :abstract_functionality, :start_line, :end_line)
+                    RETURNING function_id
+                        """
             function_id = connection.execute(
-                function_query,
+                text(self.function_query),
                 {
                     "file_id": file_id,
                     "function_name": name,
                     "function_signature": "",  # You might want to extract this from the code
                     "abstract_functionality": abstraction,
-                    "start_line": 1,  # You might want to calculate actual start and end lines
-                    "end_line": len(code.splitlines()),
+                    "start_line": start_line,
+                    "end_line": start_line + len(code.splitlines()) - 1,
                 },
             ).scalar_one()
 
@@ -302,95 +344,65 @@ class RepositoryAnalyzer:
         logger.info("Storing function relationships")
         with self.engine.begin() as connection:
             for caller, callee in self.graph.edges():
-                query = text(
+                if not self.node_query:
+                    self.node_query = """
+                        INSERT INTO node_relationships (parent_node_id, child_node_id, relationship_type)
+                        VALUES (
+                            (SELECT node_id FROM syntax_nodes WHERE node_content LIKE :caller),
+                            (SELECT node_id FROM syntax_nodes WHERE node_content LIKE :callee),
+                            'calls'
+                        )
+                        ON CONFLICT DO NOTHING
                     """
-                    INSERT INTO node_relationships (parent_node_id, child_node_id, relationship_type)
-                    VALUES (
-                        (SELECT node_id FROM syntax_nodes WHERE node_content LIKE :caller),
-                        (SELECT node_id FROM syntax_nodes WHERE node_content LIKE :callee),
-                        'calls'
-                    )
-                    ON CONFLICT DO NOTHING
-                """
-                )
+
                 connection.execute(
-                    query, {"caller": f"%{caller}%", "callee": f"%{callee}%"}
+                    text(self.node_query),
+                    {"caller": f"%{caller}%", "callee": f"%{callee}%"},
                 )
         logger.info("Function relationships stored successfully")
 
-    # def query(self, query_text: str, num_results: int = 5) -> List[Dict[str, Any]]:
-    #     logger.info(f"Performing query: '{query_text}' for repo_id: {self.repo_id}")
-
-    #     embedding, emb_usage = self.llm_provider.get_text_embedding(query_text)
-    #     embedding = embedding[0]["embedding"]
-
-    #     # Use the custom query method
-    #     results = self.vector_store.custom_query(embedding, self.repo_id, num_results)
-
-    #     processed_results = []
-    #     for result in results:
-    #         processed_results.append({
-    #             "function_name": result["metadata"].get("function_name", ""),
-    #             "abstraction": result["text"],
-    #             "file_path": result["metadata"].get("file_path", ""),
-    #             "function_signature": result["metadata"].get("function_signature", ""),
-    #             "relevance_score": result["similarity"],
-    #         })
-
-    #     logger.info(f"Query completed. Found {len(processed_results)} results.")
-    #     return processed_results
-
-    def query(self, query_text: str, num_results: int = 5) -> List[Dict[str, Any]]:
-        logger.info(f"Performing query: '{query_text}' for repo_id: {self.repo_id}")
-
-        index = VectorStoreIndex.from_vector_store(
-            self.vector_store, embed_model=self.embed_model, llm=self.llm
-        )
-
+    def query(
+        self, query_text: str, num_results: int = 5, repo_id=None
+    ) -> List[Dict[str, Any]]:
         embedding, emb_usage = self.llm_provider.get_text_embedding(query_text)
         embedding = embedding[0]["embedding"]
 
-        # Create a filter to only search within the current repository
-        # filter_dict = {"repo_id": self.repo_id}
+        results = self.vector_store.search(embedding, limit=num_results)
 
-        query_bundle = QueryBundle(query_str=query_text, embedding=embedding)
-        retriever = index.as_retriever(similarity_top_k=num_results)
+        processed_results = []
+        for result in results:
+            processed_results.append(
+                {
+                    "function_id": result.payload["function_id"],
+                    "relevance_score": result.score,
+                }
+            )
 
-        # Apply the filter during retrieval
-        nodes = retriever.retrieve(query_bundle)  # Add potential filtering
-
-        results = []
+        # Fetch additional data from the database
         with self.engine.connect() as connection:
-            for node in nodes:
-                function_id = (
-                    node.node.id_
-                )  # Assuming we stored function_id as the node id
+            for result in processed_results:
                 query = text(
                     """
                     SELECT fa.function_name, fa.abstract_functionality, f.file_path, fa.function_signature
                     FROM function_abstractions fa
                     JOIN files f ON fa.file_id = f.file_id
                     WHERE fa.function_id = :function_id
-                    """
+                """
                 )
-                result = connection.execute(
-                    query, {"function_id": function_id}
+                db_result = connection.execute(
+                    query, {"function_id": result["function_id"]}
                 ).fetchone()
-                if result:
-                    results.append(
+                if db_result:
+                    result.update(
                         {
-                            "function_name": result[0],
-                            "abstraction": result[1],
-                            "file_path": result[2],
-                            "function_signature": result[3],
-                            "relevance_score": (
-                                node.score if hasattr(node, "score") else 1.0
-                            ),
+                            "function_name": db_result[0],
+                            "abstraction": db_result[1],
+                            "file_path": db_result[2],
+                            "function_signature": db_result[3],
                         }
                     )
 
-        sorted_results = sorted(
-            results, key=lambda x: x["relevance_score"], reverse=True
+        return (
+            sorted(processed_results, key=lambda x: x["relevance_score"], reverse=True),
+            emb_usage,
         )
-        logger.info(f"Query completed. Found {len(sorted_results)} results.")
-        return sorted_results
